/*      helpinfo.h  

	part of the ldlas package
	(c) 1995, 1996 by Thor Sigvaldason
	
	This file includes routines to display help and warranty info
	
*/

void helplist (WINDOW *textbox)
{
	werase(textbox);
	wrefresh(textbox);
	wmove(textbox,0,0);
	wprintw(textbox," General Commands:\n\n");
	wprintw(textbox," about          append         bye            calculate      describe\n");
	wprintw(textbox," do             drop           exit           echo           fappend\n");
	wprintw(textbox," finput         generate       graph          histogram      in\n");
	wprintw(textbox," input          interpolate    label          let            list\n");
	wprintw(textbox," load           log            ls/dir         more           predict\n");
	wprintw(textbox," quit           refresh        regress        rename         replace\n");
	wprintw(textbox," save           setshow        summarize      texlog         tutorial\n");
	wprintw(textbox," wait           warranty\n\n");
	wprintw(textbox," Enter \"do tutorial\" (outside of Help) for an introduction to ldlas.\n");
	wprintw(textbox," Hitting \"Enter\" on a blank topic inquiry will exit help");
}		


void helpon(WINDOW *textbox,int topic)
{
	werase(textbox);
	wrefresh(textbox);
	wmove(textbox,0,0);
	switch ( topic )
	{		
		case 1  : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: bye | exit | quit \n\n");
			  wprintw(textbox," Synopsis: bye | exit | quit \n\n");
			  wprintw(textbox,"     Desc: These three commands are all equivalent, and are used to exit    \n");
			  wprintw(textbox,"           ldlas. They are only effective if there is no unsaved data in    \n");
			  wprintw(textbox,"           memory. To exit ldlas without saving data, erase it from memory  \n");
			  wprintw(textbox,"           by issuing a \"drop _all\" command first.\n");
			  wrefresh(textbox);
			  break;

		case 2  : helplist(textbox);break;

		case 3  : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: refresh           \n\n");
			  wprintw(textbox," Synopsis: refresh           \n\n");
			  wprintw(textbox,"     Desc: Because ldlas uses multiple windows in its displays of visual    \n");
			  wprintw(textbox,"           information, and because the windowing routines (ie. ncurses)    \n");
			  wprintw(textbox,"           and ldlas are imperfect software, the display can sometimes      \n");
			  wprintw(textbox,"           become confused and/or corrupted. The \"refresh\" command can be \n");
			  wprintw(textbox,"           issued to make ldlas erase and redraw all of its windows.        \n");
			  wrefresh(textbox);
			  break;

		case 4  : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: warranty          \n\n");
			  wprintw(textbox," Synopsis: warranty          \n\n");
			  wprintw(textbox,"     Desc: Used to display warranty information. The resulting display, of  \n");
			  wprintw(textbox,"           course, states that ldlas comes with no warranty whatsoever.     \n\n");
			  wprintw(textbox," See Also: about\n");
			  wrefresh(textbox);
			  break;

		case 6  : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: about\n\n");
			  wprintw(textbox," Synopsis: about\n\n");
			  wprintw(textbox,"     Desc: This command displays information about ldlas, including warranty\n");
			  wprintw(textbox,"           details, source code date, and version number.                   \n\n");
			  wprintw(textbox," See Also: warranty, tutorial");
			  wrefresh(textbox);break;

		case 7  : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: input             \n\n");
			  wprintw(textbox," Synopsis: input [ float | integer | string ] {variable_name}\n\n");
			  wprintw(textbox,"     Desc: The \"input\" command is used to physically enter data into ldlas  \n");
			  wprintw(textbox,"           from the keyboard. The system responds with numbered prompts for \n");
			  wprintw(textbox,"           each datum. Data input is terminated by hitting \"enter\" on a     \n");
			  wprintw(textbox,"           blank input prompt line. To enter integer or string data, the    \n");
			  wprintw(textbox,"           variable name must be preceeded by \"integer\" or \"string\" (res-   \n");
			  wprintw(textbox,"           pectively). Missing observations should be noted by an otherwise \n");
			  wprintw(textbox,"           unused data value (see \"help missing\" or \"help set\"). Entered    \n");
			  wprintw(textbox,"           data may be reviewed with the \"list\" command or corrected with \n");
			  wprintw(textbox,"           the \"replace\" command.\n\n");
			  wprintw(textbox," See Also: in, finput, append\n");
			  wrefresh(textbox);
			  break;
			  
		case 8  : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: list              \n\n");
			  wprintw(textbox," Synopsis: list {variable_name} [ in {start_obs} / {stop_obs} ]\n\n");
			  wprintw(textbox,"     Desc: The \"list\" command is used to display the values stored under a  \n");
			  wprintw(textbox,"           given variable name. Multiple variable names (separated by white-\n");
			  wprintw(textbox,"           space) can be supplied. If \"more\" is set to on (default), the    \n");
			  wprintw(textbox,"           listing will be stopped after each screen of information until   \n");
			  wprintw(textbox,"           the user hits a key (see \"help more\" or \"help set\"). The range \n");
			  wprintw(textbox,"           of observations displayed by the \"list\" command can be restricted \n");
			  wprintw(textbox,"           with the \"in\" option (see \"help in\").\n\n");
			  wprintw(textbox," See Also: in, more, set \n");
			  wrefresh(textbox);
			  break;
			  
		case 10 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: describe          \n\n");
			  wprintw(textbox," Synopsis: describe          \n\n");
			  wprintw(textbox,"     Desc: Unlike the \"list\" command, which displays information about the\n");
			  wprintw(textbox,"           values of specific variables, \"describe\" displays information    \n");
			  wprintw(textbox,"           about the dataset as a whole. Included in this output are the    \n");
			  wprintw(textbox,"           names and descriptions of all variables. Some further information\n");
			  wprintw(textbox,"           of this type can be accessed via the \"show\" command.           \n");
			  wprintw(textbox,"                                                                            \n\n");
			  wprintw(textbox," See Also: list, show\n");
			  wrefresh(textbox);
			  break;

		case 11 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: show              \n\n");
			  wprintw(textbox," Synopsis: show              \n\n");
			  wprintw(textbox,"     Desc: This command is used to display information about the current    \n");
			  wprintw(textbox,"           state of ldlas' storage areas. All used vectors (variables) are  \n");
			  wprintw(textbox,"           listed together with their vector number and type. The main use  \n");
			  wprintw(textbox,"           of this command from the user's (as opposed to the programmer's) \n");
			  wprintw(textbox,"           point of view is to see how much room is left to add more varia- \n");
			  wprintw(textbox,"           bles.                                                            \n\n");
			  wprintw(textbox," See Also: describe                                     \n");
			  wrefresh(textbox);
			  break;

		case 12 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: drop              \n\n");
			  wprintw(textbox," Synopsis: drop {variable_name} | _all \n\n");
			  wprintw(textbox,"     Desc: Used to remove variables from the dataset in memory. Multiple    \n");
			  wprintw(textbox,"           names can be given, or the reserved word \"_all\" may be supplied  \n");
			  wprintw(textbox,"           to remove all variables simultaneously. This latter option is    \n");
			  wprintw(textbox,"           useful for leaving ldlas without having to save the dataset in  \n");
			  wprintw(textbox,"           memory.\n\n");
			  wprintw(textbox," See Also: in, missing, bye, quit, exit   \n");
			  wrefresh(textbox);
			  break;

		case 13 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: label             \n\n");
			  wprintw(textbox," Synopsis: label data | variable {variable_name} {descriptive_string} \n\n");
			  wprintw(textbox,"     Desc: Every ldlas dataset, as well as every variable within a dataset, \n");
			  wprintw(textbox,"           can be assigned a descriptive statement. If, for example, one has\n");
			  wprintw(textbox,"           a \"gnp\" dataset consisting of time series data for a number of \n");
			  wprintw(textbox,"           years for different countries' gross domestic product, then the  \n");
			  wprintw(textbox,"           dataset might be assigned a label similar to \"label data Gross   \n");
			  wprintw(textbox,"           Domestic Poduct of Selected Countries\". A country variable within\n");
			  wprintw(textbox,"           the set could be assigned \"label variable uk United Kingdom\".  \n");
			  wprintw(textbox,"           Note that the inputed text which makes up the label can be entered\n");
			  wprintw(textbox,"           in a free-form manner (ie. it need not be surrounded by quotes or \n");
			  wprintw(textbox,"           other delimiters).\n");
			  wrefresh(textbox);
			  break;

		case 14 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: save              \n\n");
			  wprintw(textbox," Synopsis: save {legal_file_name} \n\n");
			  wprintw(textbox,"     Desc: The current dataset in memory is written to the supplied file    \n");
			  wprintw(textbox,"           name. All ldlas' save files end with a \".lds\" extension which    \n");
			  wprintw(textbox,"           need not be supplied by the user. Note that ldlas does not check \n");
			  wprintw(textbox,"           for the permissability of the supplied name and will save under  \n");
			  wprintw(textbox,"           any supplied name that the operating system allows a file to be  \n");
			  wprintw(textbox,"           opened with. Caution on the part of the user is therefore        \n");
			  wprintw(textbox,"           required to avoid over-writing existing data files. In addition  \n");
			  wprintw(textbox,"           to the variables' contents and names, all current labels and the \n");
			  wprintw(textbox,"           current value for a missing observation (see \"help missing\") are\n");
			  wprintw(textbox,"           also saved as part of the file. \n\n");
			  wprintw(textbox," See Also: load, missing, bye, quit, exit \n");
			  wrefresh(textbox);
			  break;

		case 15 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: load              \n\n");
			  wprintw(textbox," Synopsis: load {legal_file_name} \n\n");
			  wprintw(textbox,"     Desc: Used to load into memory the contents of a ldlas data file       \n");
			  wprintw(textbox,"           previously created by the \"save\" command. All ldlas data file    \n");
			  wprintw(textbox,"           names end with a \".lds\" extension which need not be supplied by  \n");
			  wprintw(textbox,"           the user. The relevant value for a missing observation (see \"help\n");
			  wprintw(textbox,"           missing\") is contained in the data file and need not be re-      \n");
			  wprintw(textbox,"           assigned via a \"set missing\" command.                            \n\n");
			  wprintw(textbox," See Also: save, missing \n");
			  wrefresh(textbox);
			  break;

		case 16 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: finput            \n\n");
			  wprintw(textbox," Synopsis: finput {file_name} [ float | integer | string ] {variable_name}\n\n");
			  wprintw(textbox,"     Desc: Similar to the \"input\" command except that multiple variables may\n");
			  wprintw(textbox,"           be specified and data is read from a space-delimited ASCII file  \n");
			  wprintw(textbox,"           rather than from the keyboard. Data within the file to be read   \n");
			  wprintw(textbox,"           must be arranged in space-delimited columns, and all string data \n");
			  wprintw(textbox,"           must have whitespace characters replaced by underscores (ie. _). \n");
			  wprintw(textbox,"           A \"file\" called \"rawdata\" of the form:                       \n");
			  wprintw(textbox,"                                                                            \n");
			  wprintw(textbox,"                                  3.6    Frank           7                  \n");
			  wprintw(textbox,"                                  5.3    Billy_Joe       3                  \n");
			  wprintw(textbox,"                                  2.1    Bob_(Robert)    6                  \n");
			  wprintw(textbox,"                                                                            \n");
			  wprintw(textbox,"           could be read with \"finput rawdata var1 string var2 integer var3\"\n\n");
			  wprintw(textbox," See Also: input\n");
			  wrefresh(textbox);
			  break;

		case 17 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: summarize         \n\n");
			  wprintw(textbox," Synopsis: summarize {variable_name}                                      \n\n");
			  wprintw(textbox,"     Desc: Produces a tabular summary of the variable in question, includ-  \n");
			  wprintw(textbox,"           ing statistics such as mean, median, and variance. If a log file \n");
			  wprintw(textbox,"           is in use, this output is logged. If a texlog file is open, a TeX\n");
			  wprintw(textbox,"           formatted table is created in that file. Other summarizing       \n");
			  wprintw(textbox,"           commands include \"histogram\".                               \n\n");
			  wprintw(textbox," See Also: histogram, log, texlog\n");
			  wrefresh(textbox);
			  break;

		case 18 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: time              \n\n");
			  wprintw(textbox," Synopsis: time                                                           \n\n");
			  wprintw(textbox,"     Desc: Displays the current time and date. If a log or texlog file is   \n");
			  wprintw(textbox,"           open, this information is also written to these files.         \n\n");
			  wprintw(textbox," See Also: log, texlog\n");
			  wrefresh(textbox);
			  break;

		case 19 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: set         \n\n");
			  wprintw(textbox," Synopsis: set [ {option} | {value} ]                                     \n\n");
			  wprintw(textbox,"     Desc: When invoked with no options, the set command displays the       \n");
			  wprintw(textbox,"           current state of all of the set options. These options and the   \n");
			  wprintw(textbox,"           values they can take are as follows:                             \n");
			  wprintw(textbox,"                                                                            \n");
			  wprintw(textbox,"              Option               Values                                 \n");
			  wprintw(textbox,"                                                                            \n");
			  wprintw(textbox,"              more                 on | off                               \n");
			  wprintw(textbox,"              missing              {signed_integer}                       \n");
			  wprintw(textbox,"              lineform             0-6 (0=invisible, 1=solid, etc.)       \n");
			  wprintw(textbox,"              pointform            0-6 (0=invisible, 1=dot, etc.)         \n");
			  wprintw(textbox,"              keyform              0-2 (0=nokey, 1=inside upper right, etc.)\n");
			  wprintw(textbox,"              graphform            {any gnuplot output format}            \n");
			  wprintw(textbox,"              keyscale             float value between 0 and 1           \n\n");
			  wprintw(textbox," See Also: missing, more, graph\n");
			  wrefresh(textbox);
			  break;

		case 21 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: log            \n\n");
			  wprintw(textbox," Synopsis: log off | on {legal_filename}                                  \n\n");
			  wprintw(textbox,"     Desc: \"log\" files are used to record parts of ldlas's terminal output  \n");
			  wprintw(textbox,"           to a (supplied) file name. A log file is begun using the \"on    \n");
			  wprintw(textbox,"           {legal_filename}\" option and turned off using the \"off\" option.\n");
			  wprintw(textbox,"           If the supplied file name already exists, the file will be       \n");
			  wprintw(textbox,"           appended to. Not all commands produce log entries, although all  \n");
			  wprintw(textbox,"           of those which generate useful information (such as \"regress\", \n");
			  wprintw(textbox,"           \"summarize\", etc.) do.\n\n");
			  wprintw(textbox," See Also: texlog\n");
			  wrefresh(textbox);
			  break;

		case 22 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: texlog          \n\n");
			  wprintw(textbox," Synopsis: texlog off | on {legal_filename}                               \n\n");
			  wprintw(textbox,"     Desc: One of ldlas's most useful features is its ability to log output \n");
			  wprintw(textbox,"           information to a \"TeX-ready\" file. Once invoked with the \"on    \n");
			  wprintw(textbox,"           {legal_filename}\" option (the user need not supply a .tex       \n");
			  wprintw(textbox,"           extension), any generated information is echoed to the file in   \n");
			  wprintw(textbox,"           tabular form. The resulting file can be then be processed by     \n");
			  wprintw(textbox,"           plain TeX, or imbedded in more complicated TeX documents.   \n\n");
			  wprintw(textbox," See Also: log, tutorial\n");
			  wrefresh(textbox);
			  break;

		case 23 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: wait         \n\n");
			  wprintw(textbox," Synopsis: wait                             \n\n");
			  wprintw(textbox,"     Desc: Used to make the user strike any key before execution continues. \n");
			  wprintw(textbox,"           This is useful in do (ie. batch) file programming.               \n\n");
			  wprintw(textbox," See Also: do, echo\n");
			  wrefresh(textbox);
			  break;

		case 25 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: echo            \n\n");
			  wprintw(textbox," Synopsis: echo {a string of text}                                        \n\n");
			  wprintw(textbox,"     Desc: In its most basic incarnation, the \"echo\" command simply echoes  \n");
			  wprintw(textbox,"           to the terminal whatever string (collection of characters) it is \n");
			  wprintw(textbox,"           followed by. As echoes are also made to log and texlog files (if\n");
			  wprintw(textbox,"           they are open), this is a handy way to insert comments between   \n");
			  wprintw(textbox,"           items of output. ldlas do (ie. batch) files can also be made much\n");
			  wprintw(textbox,"           more \"user-friendly\" with the use of imbedded echo commands.   \n\n");
			  wprintw(textbox," See Also: do, log, texlog\n");
			  wrefresh(textbox);
			  break;

		case 26 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: tutorial            \n\n");
			  wprintw(textbox," Synopsis: do tutorial\n\n");
			  wprintw(textbox,"     Desc: The online tutorial is invoked as a do file as it is not a built \n");
			  wprintw(textbox,"           in ldlas command.                                                \n\n");
			  wprintw(textbox," See Also: do\n");
			  wrefresh(textbox);
			  break;

		case 27 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: interpolate         \n\n");
			  wprintw(textbox," Synopsis: interpolate {variable_name} [ in {start_obs} {stop_obs} ]\n\n");
			  wprintw(textbox,"     Desc: This command will replace missing values in a variable with       \n");
			  wprintw(textbox,"           \"straight line\" estimates based on surrounding values. That is, \n");
			  wprintw(textbox,"           if four obervations in a variable are 2, {missing}, {missing}, 5, \n");
			  wprintw(textbox,"           then the \"interpolate\" command will calculate that the two      \n");
			  wprintw(textbox,"           missing values should be 3 and 4.                               \n\n");
			  wprintw(textbox," See Also: extrapolate\n");
			  wrefresh(textbox);
			  break;

		case 28 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: more           \n\n");
			  wprintw(textbox," Synopsis: set more on | off                    \n\n");
			  wprintw(textbox,"     Desc: The \"more\" option is handled via the \"set\" command. It used to   \n");
			  wprintw(textbox,"           turn on and off the output being paused after each full screen of\n");
			  wprintw(textbox,"           information. It is generally set \"on\" for interactive work and   \n");
			  wprintw(textbox,"           turned off for batch (do) files.\n\n");
			  wprintw(textbox," See Also: do, set\n");
			  wrefresh(textbox);
			  break;

		case 29 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: missing           \n\n");
			  wprintw(textbox," Synopsis: set missing {integer_value}             \n\n");
			  wprintw(textbox,"     Desc: The \"missing\" option is handled via the \"set\" command. It is used\n");
			  wprintw(textbox,"           to determine the value that ldlas will consider as a missing     \n");
			  wprintw(textbox,"           value. The default value is -999999.                             \n\n");
			  wprintw(textbox," See Also: set\n");
			  wrefresh(textbox);
			  break;

		case 30 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: ls | dir             \n\n");
			  wprintw(textbox," Synopsis: ls | dir [valid_ls_options] [directory_path]\n\n");
			  wprintw(textbox,"     Desc: This will generate a directory listing just as though the user   \n");
			  wprintw(textbox,"           had entered the same command at a shell prompt.                \n\n");
			  wprintw(textbox," See Also: shell\n");
			  wrefresh(textbox);
			  break;
		case 31 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: in             \n\n");
			  wprintw(textbox," Synopsis: {command} [ in {start_obs} {stop_obs} ]\n\n");
			  wprintw(textbox,"     Desc: The \"in\" option is used to restrict the use of certain ldlas   \n");
			  wprintw(textbox,"           commands to a limited number of observations specified by lower  \n");
			  wprintw(textbox,"           and upper bounds                                                 \n\n");
			  wprintw(textbox," See Also: graph, histogram, list, regress, summarize\n");
			  wrefresh(textbox);
			  break;

		case 32 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: do             \n\n");
			  wprintw(textbox," Synopsis: do {ldlas_batchfile}                 \n\n");
			  wprintw(textbox,"     Desc: The \"do\" command is used to switch execution of commands from  \n");
			  wprintw(textbox,"           the terminal to a text file which consists of ldlas commands. For\n");
			  wprintw(textbox,"           details of the ldlas batch language, please see the manual.      \n\n");
			  wprintw(textbox," See Also: echo, query\n");
			  wrefresh(textbox);
			  break;

		case 33 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: fappend        \n\n");
			  wprintw(textbox," Synopsis: fappend  {ldlas_save_file}          \n\n");
			  wprintw(textbox,"     Desc: This command is used to add data from an existing .lds file to   \n");
			  wprintw(textbox,"           the dataset in memory. Any variables in the loaded file which    \n");
			  wprintw(textbox,"           have the same name as existing variables will cause the new data \n");
			  wprintw(textbox,"           to be added to the end of the existing variable. Those with \"new\"\n");
			  wprintw(textbox,"           names will result in new variables being created.                \n\n");
			  wprintw(textbox," See Also: append, finput, input \n");
			  wrefresh(textbox);
			  break;

		case 34 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: histogram      \n\n");
			  wprintw(textbox," Synopsis: histogram {ldlas_variable} [ in {start_obs} {stop_obs} ] \n");
			  wprintw(textbox,"           [ options ] \n\n");
			  wprintw(textbox,"     Desc: Used to generate a histogram plot of a single variables contents.\n");
			  wprintw(textbox,"           Options include \"bars(x)\" (to set the number of groups the data is\n");
			  wprintw(textbox,"           broken into) and \"save(filename})\" (where \"filename\" is a file to\n");
			  wprintw(textbox,"           which the graph should be written. The save format of the graph  \n");
			  wprintw(textbox,"           (as well as many other graph options) is determined by the \"set\"\n");
			  wprintw(textbox,"           command.                                                        \n\n");
			  wprintw(textbox," See Also: graph, set\n");
			  wrefresh(textbox);
			  break;

		case 36 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: append         \n\n");
			  wprintw(textbox," Synopsis: append {variable_name}               \n\n");
			  wprintw(textbox,"     Desc: Used to manually input extra observations and the end of exist-  \n");
			  wprintw(textbox,"           ing variables. To add observations already stored in a .lds file \n");
			  wprintw(textbox,"           use \"fappend\".                                                 \n\n");
			  wprintw(textbox," See Also: fappend, finput, input\n");
			  wrefresh(textbox);
			  break;

/*		case 37 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: nnp            \n\n");
			  wprintw(textbox," Synopsis: nnp {variable_name} {new_variable} {obs_to_predict} \n");
			  wprintw(textbox,"           [ in {start_obs} {stop_obs} ]\n\n");
			  wprintw(textbox,"     Desc: This routine uses a neural network algorithm to predict    \n");
			  wprintw(textbox,"           \"future\" values of a given variable based on existing values. For\n");
			  wprintw(textbox,"           details, please see the manual.                                  \n\n");
			  wprintw(textbox," See Also: extrapolate, predict \n");
			  wrefresh(textbox);
			  break;*/

		case 39 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: regress        \n\n");
			  wprintw(textbox," Synopsis: regress {dependent_variable} {independent variable(s)} \n");
			  wprintw(textbox,"           [ in {start_obs} {stop_obs} ]\n\n");
			  wprintw(textbox,"     Desc: Standard Ordinary Least Squares regression which generates a     \n");
			  wprintw(textbox,"           table of output including coefficient estimates, t statistics,   \n");
			  wprintw(textbox,"           and R-squared values. Output is echoed to existing log and/or    \n");
			  wprintw(textbox,"           texlog files.\n\n");
			  wprintw(textbox," See Also: predict\n");
			  wrefresh(textbox);
			  break;

/*		case 40 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: extrapolate      \n\n");
			  wprintw(textbox," Synopsis: extrapolate {variable_name} {number_lags} {new_variable}\n");
			  wprintw(textbox,"           {number_obs} [ in {start_obs} {stop_obs} ]\n\n");
			  wprintw(textbox,"     Desc: This command assumes that the variable in question has been      \n");
			  wprintw(textbox,"           generated by an AR(number_lags) process (see manual) and         \n");
			  wprintw(textbox,"           extrapolates this process into \"future\" observations.          \n\n");
			  wprintw(textbox," See Also: nnp, predict\n");
			  wrefresh(textbox);
			  break;*/

		case 41 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: generate       \n\n");
			  wprintw(textbox," Synopsis: generate {new_variable} = {valid expression}\n\n");
			  wprintw(textbox,"     Desc: The \"generate\" command allows the user to perform mathematical \n");
			  wprintw(textbox,"           operations on existing values to create new variables. Items     \n");
			  wprintw(textbox,"           which can be used in a {valid expression} include standard \n");
			  wprintw(textbox,"           operations (+ - * / etc.), other variables, and constants.       \n\n"); 
			  wprintw(textbox," See Also: let, replace\n");
			  wrefresh(textbox);
			  break;

		case 42 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: predict         \n\n");
			  wprintw(textbox," Synopsis: predict {new_variable} [ in {start_obs} {stop_obs} ]   \n\n");
			  wprintw(textbox,"     Desc: The predict command can be used to generate a variable of fitted \n");
			  wprintw(textbox,"           values based on the results of the most recent regression.     \n\n");
			  wprintw(textbox," See Also: regress\n");
			  wrefresh(textbox);
			  break;

		case 43 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: rename           \n\n");
			  wprintw(textbox," Synopsis: rename {variable_name} {new_name} \n\n");
			  wprintw(textbox,"     Desc: Used to change the name of a variable from {variable_name} to    \n");
			  wprintw(textbox,"           {new_name}.                                                    \n\n");
			  wrefresh(textbox);
			  break;

/*		case 45 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: query          \n\n");
			  wprintw(textbox," Synopsis: query ( {set_of_chars} ) ( {char(s)} : {command(s)} )    \n\n");
			  wprintw(textbox,"     Desc: Used almost exclusively in do (ie. batch) files, the \"query\"   \n");
			  wprintw(textbox,"           command waits for the user to enter one of a defined set of      \n");
			  wprintw(textbox,"           characters and then execute commands based on the choice made:   \n");
			  wprintw(textbox,"                                                                            \n");
			  wprintw(textbox,"                       echo Do you want to (i)nterpolate or (e)xtrapolate?  \n");
			  wprintw(textbox,"                       query ( ie )                                         \n");
			  wprintw(textbox,"                             ( i : interpolate var1                         \n");
			  wprintw(textbox,"                               e : extrapolate var1 var2 100  )               \n");
			  wprintw(textbox,"                                                                            \n");
			  wprintw(textbox,"           The above will carry out the given commands based on whether the \n");
			  wprintw(textbox,"           user enters \"i\" or \"e\".                                    \n\n");
			  wprintw(textbox," See Also: do \n");
			  wrefresh(textbox);
			  break;*/

		case 47 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: graph         \n\n");
			  wprintw(textbox," Synopsis: graph {y_variable(s)} {x_variable} [ in {start_obs} {stop_obs} ] \n");
			  wprintw(textbox,"           [ , options ]\n\n");
			  wprintw(textbox,"     Desc: This command will plot any number of vertical-axis variables     \n");
			  wprintw(textbox,"           against a single horizontal-axis variable with a variety of      \n");
			  wprintw(textbox,"           plotting options (see manual).                                  \n\n");
			  wprintw(textbox," See Also: histogram \n");
			  wrefresh(textbox);
			  break;

		case 48 : wprintw(textbox,"\n\n");
		          wprintw(textbox,"    Topic: let            \n\n");
			  wprintw(textbox," Synopsis: let _{constant_name} = {valid expression}\n\n");
			  wprintw(textbox,"     Desc: ldlas has up to 256 user defined constant values which are assig-\n");
			  wprintw(textbox,"           ned via the \"let\" command. Constant names must begin with an   \n");
			  wprintw(textbox,"           underscore character to distinguish them from regular variables. \n");
			  wprintw(textbox,"           Note that the constant  \"_n\" is predefined to be equal to the  \n");
			  wprintw(textbox,"           current observation number.\n");
			  wrefresh(textbox);
			  break;

		default : helplist(textbox);break;
	}
}

void help (char *cstring)
{
	WINDOW *outline;
	WINDOW *textbox;
	char helptopic[MAX_INPUT];
	int escapehelp = 0;
	echo();
	saystatus("Help Mode          ");
	outline=newwin(LINES-1,COLS,0,0);
	wstandout(outline);
	box(outline,0,0);
	mvwaddstr(outline,0,(COLS/2)-3,"HELP");
	wstandend(outline);
	wrefresh(outline);
	textbox=newwin(LINES-3,COLS-2,1,1);
	scrollok(textbox,TRUE);
	strcpy(helptopic,puller(cstring));
	if(strlen(helptopic) < 1) 
	{
		helplist(textbox);
		wmove(textbox,LINES-4,0);
		wprintw(textbox,"    Topic ? ");
		wrefresh(textbox);
		wscanw(textbox,"%s",&helptopic);
		wrefresh(textbox);
		if(strlen(helptopic) < 1) escapehelp = 1;
	}
	while(escapehelp == 0)
	{
		helpon(textbox, parse_input(puller(helptopic)));
		wmove(textbox,LINES-4,0);
		wprintw(textbox,"  Topic ? ");
		wrefresh(textbox);
		wscanw(textbox,"%s",&helptopic);
		wrefresh(textbox);
		if(strlen(helptopic) < 1) escapehelp = 1;
		touchwin(outline);
		wrefresh(outline);
		touchwin(textbox);
		wrefresh(textbox);
		wrefresh(outline);
	}		
	wclear(textbox);
	wclear(outline);
	wrefresh(textbox);
	wrefresh(outline);	
	delwin(textbox);
	delwin(outline);
	noecho();
	refresh();
}	

void warranty(void)
{
	wprintw(ioarea,"\n\n      ldlas comes with no warranty, implied or otherwise; use at your own risk.\n");
	wrefresh(ioarea);
	if(strlen(puller(coinput)) > 0) small_error(3);
}

void about(void)
{
   wprintw (ioarea,"\n\n");
   wrefresh(ioarea);
   wprintw (ioarea, "         ldlas is a program designed to handle data, perform statistical  \n");
   wrefresh(ioarea);
   wprintw (ioarea, "      operations, and produce graphical output via gnuplot. This program  \n");
   wrefresh(ioarea);
   wprintw (ioarea, "      is distributed free under the condition that no fee may be charged  \n");
   wrefresh(ioarea);
   wprintw (ioarea, "      for redistribution. ldlas comes with no warranty either implied or  \n");
   wrefresh(ioarea);
   wprintw (ioarea, "      intended (use at your own risk). For an online introduction, enter  \n");
   wrefresh(ioarea);
   wprintw (ioarea, "      \"do tutorial\"  at the command prompt.  Information on commands can \n");
   wrefresh(ioarea);
   wprintw (ioarea, "      be found in the manual (see /usr/lib/ldlas/doc/) or via the \"help\" \n");
   wprintw (ioarea, "      command.  \n");
   wrefresh(ioarea);
   wprintw (ioarea, "                                                                          \n");
   wrefresh(ioarea);
   wprintw (ioarea, "               Program Name: ldlas                                       \n");
   wrefresh(ioarea);
   wprintw (ioarea, "        Explanation of Name: Acronym of ldlas, damn lies, and statistics \n");
   wrefresh(ioarea);
   wprintw (ioarea, "                     Author: Thor Sigvaldason                            \n");
   wrefresh(ioarea);
   wprintw (ioarea, "           Author's Address: T.Sigvaldason@lse.ac.uk                     \n");
   wrefresh(ioarea);
   wprintw (ioarea, "             Version Number: %.1f                              \n",LDLAS_VER);
   wrefresh(ioarea);
   wprintw (ioarea, "           Source Code Date: %s                               \n",LDLAS_DATE);
   wrefresh(ioarea);
   wprintw (ioarea, " \n");
   wrefresh(ioarea);
   if(strlen(puller(coinput)) > 0) small_error(3);
}

void why(void)
{
   wprintw (ioarea,"\nBecause.");
   wrefresh(ioarea);
}


