/*
        ldataio.h

	part of the ldlas package
	(c) 1995, 1996 by Thor Sigvaldason
	
	This file handles all of the data input and output:
	
		d_input()  allows the user to input data
		f_input()  allows the user to load space delimited data
		dsave()   allows the user to save in .lds format
		dload()   allows the user to load an .lds format file
		dlog()    opens a normal log file
		dtlog()   opens a tex log file
*/

void flushempty (void)
{
	int count,check = 0;
	for(count = 0; count < MAX_VARS; count++)
	{
		if(d_matrix[count].nobs == 0)
		{
			strcpy(d_matrix[count].v_name,"");
			strcpy(d_matrix[count].v_desc,"");
			if(d_matrix[count].fnumb != -1)
			{
				free(fvec[d_matrix[count].fnumb]);
				d_matrix[count].fnumb = -1;
			}
			if(d_matrix[count].inumb != -1)
			{
				free(ivec[d_matrix[count].inumb]);
				d_matrix[count].inumb = -1;
			}
			d_matrix[count].snumb = -1;
		}
		else
		{
			check = 1;
		}
	}
	for(count = 0; count < MAX_CONS; count++)
	{
		if(strlen(constants[count].c_name) > 0) check = 1;
	}
	if(check==0) oktoquit = TRUE;
	if(check==1) oktoquit = FALSE;	
}

void f_input(char vname[MAX_INPUT])
{
	int vnum,f_numb,count,escapeinput,check;
	char ininput[MAX_INPUT];
	char **waisted;
	count = 0;
	escapeinput=0;
	vnum = findempty();
	if(vnum < 0)
	{
		alert(error(3),alert_time);
		return;
	}
	f_numb = typeaug(1,vnum);
	fvec[f_numb] = (double *) malloc (MAX_OBS * sizeof(double));
	if (!fvec[f_numb])
	{
		alert(error(4),alert_time);
		return;
	}
	strcpy(d_matrix[vnum].v_name,vname);
	d_matrix[vnum].fnumb = f_numb;
	d_matrix[vnum].fmem_start=fvec[f_numb];
	wprintw(ioarea,"\nEnter floating point values for %s\n",d_matrix[vnum].v_name);
	do
	{
		wprintw(ioarea,"\n%5i>",(count+1));
		wrefresh(ioarea);
		get_input(ininput);
		if (strcmp(ininput,"\0") == 0) escapeinput = 1;
		else
		{
			*fvec[f_numb] = atof(ininput);
			fvec[f_numb]++;
			count = count + 1;
		}
	}
	while(escapeinput == 0);
	d_matrix[vnum].nobs = count;
	flushempty();
	wprintw(ioarea,"\n");
	wrefresh(ioarea);
}

void i_input(char vname[MAX_INPUT])
{
	int vnum,i_numb,count,escapeinput;
	char ininput[MAX_INPUT];
	char **waisted;
	count = 0;
	escapeinput=0;
	vnum = findempty();
	if(vnum < 0)
	{
		alert(error(3),alert_time);
		return;
	}
	i_numb = typeaug(2,vnum);
	ivec[i_numb] = (int *) malloc (MAX_OBS * sizeof(int));
	if (!ivec[i_numb])
	{
		alert(error(4),alert_time);
		return;
	}
	strcpy(d_matrix[vnum].v_name,vname);
	d_matrix[vnum].inumb = i_numb;
	d_matrix[vnum].imem_start=ivec[i_numb];
	wprintw(ioarea,"\nEnter integer values for %s\n",d_matrix[vnum].v_name);
	do
	{
		wprintw(ioarea,"\n%5i>",(count+1));
		wrefresh(ioarea);
		get_input(ininput);
		if (strcmp(ininput,"\0") == 0) escapeinput = 1;
		else
		{
			*ivec[i_numb] = strtol(ininput,waisted,0);
			ivec[i_numb]++;
			count = count + 1;
		}
	}
	while(escapeinput == 0);
	d_matrix[vnum].nobs = count;
	flushempty();
	wprintw(ioarea,"\n");
	wrefresh(ioarea);
}

void s_input(char vname[MAX_INPUT])
{
	int vnum,s_numb,count,escapeinput;
	char ininput[MAX_INPUT];
	count = 0;
	escapeinput=0;
	vnum = findempty();
	if(vnum < 0)
	{
		alert(error(3),alert_time);
		return;
	}
	s_numb = typeaug(3,vnum);
	if(s_numb + 1 > MAX_SVARS)
	{
		alert(error(5),alert_time);
		return;
	}
	strncpy(d_matrix[vnum].v_name,vname,20);
	d_matrix[vnum].snumb = s_numb;
	wprintw(ioarea,"\nEnter character strings for %s\n",d_matrix[vnum].v_name);
	do
	{
		wprintw(ioarea,"\n%5i>",(count+1));
		wrefresh(ioarea);
		get_input(ininput);
		if (strcmp(ininput,"\0") == 0) escapeinput = 1;
		else
		{
			strcpy(smatrix[s_numb][count],ininput);
			count++;
		}
	}
	while(escapeinput == 0);
	d_matrix[vnum].nobs = count;
	flushempty();
	wprintw(ioarea,"\n");
	wrefresh(ioarea);
}

int checkname(char *vname)
{
	int count,check;
	check = 0;
	for(count = 0;count < MAX_VARS; count ++)
	{
		if(strncmp(d_matrix[count].v_name,vname,19) == 0) check = -1;
	}
	return(check);
}

int findempty(void)
{
	int count = 0;
	do
	{
	if(d_matrix[count].fnumb==-1 
	&& d_matrix[count].inumb==-1
	&& d_matrix[count].snumb==-1) return count;
	count++;
	}
	while (count < MAX_VARS);
	return -1;
}

int typeaug(int type,int vnum)
{
	int count = 0;
	int augm =  0;
	if(type==1)
	{
		for(count = 0;count < vnum; count++)
		{
			if(d_matrix[count].fnumb != -1) augm++;
		}
		return(augm);
	}
	if(type==2)
		{
		for(count = 0;count < vnum; count++)
		{
			if(d_matrix[count].inumb != -1) augm++;
		}
		;return(augm);
	}
	if(type==3)
	{
		for(count = 0;count < vnum; count++)
		{
			if(d_matrix[count].snumb != -1) augm++;
		}
		return(augm);
	}
	alert(error(42),alert_time);
}

void d_input (char coinput[MAX_INPUT])
{
	int escapeinput = 0;
	int count = 0;
	char oput[MAX_INPUT];
	char tput[MAX_INPUT];
	saystatus("Data Input Mode   ");
	strncpy(oput,puller(coinput),20);
	if(strlen(oput) < 1)
	{
		alert(error(2), alert_time);
		return;
	}
	if(oput[0] < 65 || oput[0] > 122)
	{
		alert(error(53), alert_time);
		return;
	}
	if(oput[0] > 90 && oput[0] < 97)
	{
		alert(error(53), alert_time);
		return;
	}
	strncpy(tput,puller(coinput),20);
	if(strncmp("float",oput,5)==0)
	{
		if(strlen(tput) < 1)
		{
			alert(error(15),alert_time);
			return;
		}
		if(checkname(tput) < 0)
		{
			alert(error(14),alert_time);
			return;
		}
		if(atof(tput) != 0.0 || strncmp(tput,"0",1) == 0)
		{
			alert(error(19),alert_time);
			return;
		}
		f_input(tput);
		return;
	}
	if(strncmp("string",oput,5)==0)
	{
		if(strlen(tput) < 1)
		{
			alert(error(15),alert_time);
			return;
		}
		if(checkname(tput) < 0)
		{
			alert(error(14),alert_time);
			return;
		}
		if(atof(tput) != 0.0 || strncmp(tput,"0",1) == 0)
		{
			alert(error(19),alert_time);
			return;
		}
		s_input(tput);
		return;
	}
	if(strncmp("integer",oput,7)==0)
	{
		if(strlen(tput) < 1)
		{
			alert(error(15),alert_time);
			return;
		}
		if(checkname(tput) < 0)
		{
			alert(error(14),alert_time);
			return;
		}
		if(atof(tput) != 0.0 || strncmp(tput,"0",1) == 0)
		{
			alert(error(19),alert_time);
			return;
		}
		i_input(tput);
		return;
	}
	if(strlen(oput) >= 1)
	{
		if(checkname(oput) < 0)
		{
			alert(error(14),alert_time);
			return;
		}
		if(atof(oput) != 0.0 || strncmp(oput,"0",1) == 0)
		{
			alert(error(19),alert_time);
			return;
		}
		f_input(oput);
		return;
	}
	alert(error(42),alert_time);
}	 

void dropall (void)
{
	int count;
	for(count = 0; count < MAX_VARS; count++)
	{
		d_matrix[count].nobs = 0;
	}
	for(count = 0; count < MAX_CONS; count++)
	{
		strcpy(constants[count].c_name,"\0");
	}
	strcpy(d_matrix_desc,"");
	flushempty();
	oktoquit = TRUE;
}

void droproutine (char coinput[MAX_INPUT])
{
	int count, escapedrop, todrop;
	char vname[NAME_LENGTH];
	escapedrop = 0;
	if(strlen(coinput) < 1)
	{
		alert(error(47),alert_time);
		return;
	}
	do
	{
		strcpy(vname,puller(coinput));
		if(strlen(vname) < 1)
		{
			escapedrop = 1;
		}
		if(strcmp(vname,"_all") == 0)
		{
			dropall();
			return;
		}
		todrop = 0;
		for(count = 0; count < MAX_VARS; count ++)
		{
			if(strcmp(d_matrix[count].v_name,vname)==0) 
			{
				d_matrix[count].nobs = 0;
				todrop = 1;
			}
		}
		if(todrop == 0) alert(error(13),alert_time);
	}
	while(escapedrop == 0);
	return;		
}

void ddrop(char coinput[MAX_INPUT])
{
	droproutine(coinput);
	flushempty();
}

void label(char coinput[MAX_INPUT])
{
	char cstring[MAX_INPUT];
	char tempstring[MAX_INPUT];
	int count, check;
	check = 0;
	strcpy(cstring,puller(coinput));
	if(strncmp(cstring,"dat",3) == 0)
	{
		strcpy(tempstring,puller(coinput));
		strcat(tempstring,coinput);
		strcpy(coinput,tempstring);
		strncpy(d_matrix_desc,coinput,40);
		return;
	}
	if(strncmp(cstring,"var",3) == 0)
	{
		strcpy(cstring,puller(coinput));
		for(count = 0; count < MAX_VARS; count++)
		{
			if(strcmp(cstring,d_matrix[count].v_name) == 0) 
			{
				strcpy(tempstring,puller(coinput));
				strcat(tempstring,coinput);
				strcpy(coinput,tempstring);
				strncpy(d_matrix[count].v_desc,coinput,40);
				check = 1;
			}
		}
	}
	if(check != 1) alert(error(7),alert_time);
	return;
}

char *remove_blanks (char *inputstring)
{
	char working[MAX_INPUT];
	int count;
	strcpy(working,inputstring);
	for(count = 0; count < MAX_INPUT - 1; count ++)
	{
		if(working[count] == ' ') working[count] = '_';
	}
	strcpy(inputstring,working);
	return(inputstring);
}

char *add_blanks (char *inputstring)
{
	char working[MAX_INPUT];
	int count;
	strcpy(working,inputstring);
	for(count = 0; count < MAX_INPUT - 1; count++)
	{
		if(working[count] == '_') working[count] = ' ';
	}
	strcpy(inputstring,working);
	return(inputstring);
}

void dsave(char cstring[MAX_INPUT])
{
	FILE *fptr;
	int count,countt;
	char filename[NAME_LENGTH];
	saystatus("Saving Data        ");
	strcpy(filename,puller(cstring));
	if(strlen(filename) < 1)
	{
		alert(error(48), alert_time);
		return;
	}
	if(!strchr(filename,'.')) strcat(filename,".lds");
	fptr = fopen(filename,"w");
	if(!fptr)
	{
		alert(error(8),alert_time);
		return;
	}
	strcpy(d_matrix_desc,remove_blanks(d_matrix_desc));
	if(strlen(d_matrix_desc) < 1) strcpy(d_matrix_desc,"_\0");
	for(count = 0; count <= MAX_VARS; count++)
	{
		strcpy(d_matrix[count].v_desc,remove_blanks(d_matrix[count].v_desc));
		if(strlen(d_matrix[count].v_desc) < 1) strcpy(d_matrix[count].v_desc,"_\0");
	}
	fprintf(fptr,"%s\n",d_matrix_desc);
	fprintf(fptr,"%i\n",missing_ob);
	for(count = 0; count < MAX_VARS; count++)
	{
		if(d_matrix[count].nobs > 0)
		{
			fprintf(fptr,"%-20s\n",reporttype(count));
			fprintf(fptr,"%-20s\n",d_matrix[count].v_name);
			fprintf(fptr,"%-40s\n",d_matrix[count].v_desc);
			fprintf(fptr,"%i\n",d_matrix[count].nobs);
			if(d_matrix[count].fnumb != -1)
			{
				fvec[d_matrix[count].fnumb]=d_matrix[count].fmem_start;
				for(countt = 0; countt < d_matrix[count].nobs; countt++)
				{
					fprintf(fptr,"%lf\n",*fvec[d_matrix[count].fnumb]);
					fvec[d_matrix[count].fnumb]++;
				}
			}
			if(d_matrix[count].inumb != -1)
			{
				ivec[d_matrix[count].inumb]=d_matrix[count].imem_start;
				for(countt = 0; countt < d_matrix[count].nobs; countt++)
				{
					fprintf(fptr,"%i\n",*ivec[d_matrix[count].inumb]);
					ivec[d_matrix[count].inumb]++;
				}
			}
			if(d_matrix[count].snumb != -1)
			{
				for(countt = 0; countt < d_matrix[count].nobs; countt++)
				{
					fprintf(fptr,"%-20s\n",remove_blanks(smatrix[d_matrix[count].snumb][countt]));
				}
			}
		}
	}
	for(count = 0; count < MAX_CONS; count ++)
	{
		if(strlen(constants[count].c_name) > 0)
		{
			fprintf(fptr,"%-20s\n","constant");
			fprintf(fptr,"%-20s\n",constants[count].c_name);
			fprintf(fptr,"%f\n",constants[count].c_value);
		}
	}
	fclose(fptr);
	strcpy(d_matrix_desc,add_blanks(d_matrix_desc));
	for(count = 0; count <= MAX_VARS; count++)
	{
		strcpy(d_matrix[count].v_desc,add_blanks(d_matrix[count].v_desc));
	}
	oktoquit=TRUE;
	if(strlen(puller(coinput)) > 0) small_error(3);
}

void getafloat(FILE *lfptr, char *varname)
{
	int count,vnumb,f_numb;
	vnumb=findempty();
	if(vnumb < 0)
	{
		alert(error(13), alert_time);
		return;
	}
	f_numb=typeaug(1,vnumb);
	fvec[f_numb] = (double *) malloc (MAX_OBS * sizeof(double));
	if (!fvec[f_numb])
	{
		alert(error(4),alert_time);
		return;
	}
	d_matrix[vnumb].fnumb = f_numb;
	strcpy(d_matrix[vnumb].v_name, varname);
	fscanf(lfptr,"%s\n",&d_matrix[vnumb].v_desc);
	fscanf(lfptr,"%i\n",&d_matrix[vnumb].nobs);
	d_matrix[vnumb].fmem_start=fvec[f_numb];
	for(count = 0; count < d_matrix[vnumb].nobs; count ++)
	{
		fscanf(lfptr,"%lf\n",&*fvec[f_numb]);
		fvec[f_numb]++;
	}
}
		
void getainteger(FILE *lfptr, char *varname)
{
	int count,vnumb,i_numb;
	vnumb=findempty();
	if(vnumb < 0)
	{
		alert(error(13),alert_time);
		return;
	}
	i_numb=typeaug(2,vnumb);
	ivec[i_numb] = (int *) malloc (MAX_OBS * sizeof(int));
	if (!ivec[i_numb])
	{
		alert(error(4),alert_time);
		return;
	}
	d_matrix[vnumb].inumb = i_numb;
	strcpy(d_matrix[vnumb].v_name, varname);
	fscanf(lfptr,"%s\n",&d_matrix[vnumb].v_desc);
	fscanf(lfptr,"%i\n",&d_matrix[vnumb].nobs);
	d_matrix[vnumb].imem_start=ivec[i_numb];
	for(count = 0; count < d_matrix[vnumb].nobs; count ++)
	{
		fscanf(lfptr,"%i\n",&*ivec[i_numb]);
		ivec[i_numb]++;
	}
}

void getastring(FILE *lfptr, char *varname)
{
	int count,vnumb,s_numb;
	char finput[MAX_INPUT];
	vnumb=findempty();
	if(vnumb < 0)
	{
		alert(error(13),alert_time);
		return;
	}
	s_numb=typeaug(3,vnumb);
	if(s_numb + 1 > MAX_SVARS)
	{
		alert(error(5),alert_time);
		return;
	}
	d_matrix[vnumb].snumb = s_numb;
	strcpy(d_matrix[vnumb].v_name, varname);
	fscanf(lfptr,"%s\n",&d_matrix[vnumb].v_desc);
	fscanf(lfptr,"%i\n",&d_matrix[vnumb].nobs);
	for(count = 0; count < d_matrix[vnumb].nobs; count ++)
	{
		fscanf(lfptr,"%s\n",&finput);
		strcpy(finput,add_blanks(finput));
		strcpy(smatrix[s_numb][count],finput);
	}
}

void appafloat(FILE *lfptr, int vnumb)
{
	int count,f_numb;
	char nothing[MAX_INPUT];
	int howlong;
	f_numb=d_matrix[vnumb].fnumb;
	if(f_numb == -1)
	{
		alert(error(60), alert_time);
		fclose(lfptr);
		return;
	}
	fvec[f_numb] = d_matrix[vnumb].fmem_start;
	for(count=0; count < d_matrix[vnumb].nobs;count++)
	{
		fvec[f_numb]++;
	}
	fscanf(lfptr,"%s\n",&nothing);
	fscanf(lfptr,"%i\n",&howlong);
	if(d_matrix[vnumb].nobs + howlong + 1 > MAX_OBS)
	{
		alert(error(59), alert_time);
		howlong = MAX_OBS - d_matrix[vnumb].nobs - 1;
	}
	for(count = 0; count < howlong; count ++)
	{
		fscanf(lfptr,"%lf\n",&*fvec[f_numb]);
		fvec[f_numb]++;
	}
	d_matrix[vnumb].nobs = d_matrix[vnumb].nobs + howlong;
}

void appainteger(FILE *lfptr, int vnumb)
{
	int count,i_numb;
	char nothing[MAX_INPUT];
	int howlong;
	i_numb=d_matrix[vnumb].inumb;
	if(i_numb == -1)
	{
		alert(error(60), alert_time);
		fclose(lfptr);
		return;
	}
	ivec[i_numb] = d_matrix[vnumb].imem_start;
	for(count=0; count < d_matrix[vnumb].nobs;count++)
	{
		ivec[i_numb]++;
	}
	fscanf(lfptr,"%s\n",&nothing);
	fscanf(lfptr,"%i\n",&howlong);
	if(d_matrix[vnumb].nobs + howlong + 1 > MAX_OBS)
	{
		alert(error(59), alert_time);
		howlong = MAX_OBS - d_matrix[vnumb].nobs - 1;
	}
	for(count = 0; count < howlong; count ++)
	{
		fscanf(lfptr,"%i\n",&*ivec[i_numb]);
		ivec[i_numb]++;
	}
	d_matrix[vnumb].nobs = d_matrix[vnumb].nobs + howlong;
}

void appastring(FILE *lfptr, int vnumb)
{
	int count,s_numb;
	char finput[MAX_INPUT];
	char nothing[MAX_INPUT];
	int howlong;
	s_numb=d_matrix[vnumb].snumb;
	if(s_numb == -1)
	{
		alert(error(60), alert_time);
		fclose(lfptr);
		return;
	}
	if(s_numb + 1 > MAX_SVARS)
	{
		alert(error(5),alert_time);
		return;
	}
	fscanf(lfptr,"%s\n",&nothing);
	fscanf(lfptr,"%i\n",&howlong);
	if(d_matrix[vnumb].nobs + howlong + 1 > MAX_OBS)
	{
		alert(error(59), alert_time);
		howlong = MAX_OBS - d_matrix[vnumb].nobs - 1;
	}
	for(count = d_matrix[vnumb].nobs; count < d_matrix[vnumb].nobs + howlong; count ++)
	{
		fscanf(lfptr,"%s\n",&finput);
		strcpy(finput,add_blanks(finput));
		strcpy(smatrix[s_numb][count],finput);
	}
	d_matrix[vnumb].nobs = d_matrix[vnumb].nobs + howlong;
}

void getavar(FILE *lfptr)
{
	int count,vnumb,f_numb,i_numb,s_numb;
	char finput[MAX_INPUT], temp[MAX_INPUT];
	char **waisted;
	float con_value;
	fscanf(lfptr,"%s\n",&finput);
	fscanf(lfptr,"%s\n",&temp);
	if(strncmp(finput,"float",3)==0)
	{
		getafloat(lfptr, temp);
		return;
	}
	if(strncmp(finput,"integer",3)==0)
	{
		getainteger(lfptr, temp);
		return;
	}
	if(strncmp(finput,"string",3)==0)
	{
		getastring(lfptr, temp);
		return;
	}
	if(strncmp(finput,"constant",3)==0)
	{
		fscanf(lfptr,"%f\n",&con_value);
		for(count = 0; count < MAX_CONS; count ++)
		{
			if(strlen(constants[count].c_name) < 1)
			{
				strcpy(constants[count].c_name,temp);
				constants[count].c_value = con_value;
				count = MAX_CONS + 1;
			}
		}
		if(count < MAX_CONS + 1) alert(error(52), alert_time);
		return;
	}
	alert(error(10), 1);	
}

void appavar(FILE *lfptr)
{
	int count,countt,vnumb,f_numb,i_numb,s_numb, howmany, match;
	char finput[MAX_INPUT], temp[MAX_INPUT];
	char **waisted;
	float con_value;
	fscanf(lfptr,"%s\n",&finput);
	if(strncmp(finput,"float",3)==0)
	{
		fscanf(lfptr,"%s\n",&temp);
		for(count = 0; count < MAX_VARS; count++)
		{
			if(strcmp(d_matrix[count].v_name,temp) == 0)
			{
				appafloat(lfptr, count);
				return;
			}
		}
		getafloat(lfptr, temp);
		return;
	}
	if(strncmp(finput,"integer",3)==0)
	{
		fscanf(lfptr,"%s\n",&temp);
		for(count = 0; count < MAX_VARS; count++)
		{
			if(strcmp(d_matrix[count].v_name,temp) == 0)
			{
				appainteger(lfptr, count);
				return;
			}
		}
		getainteger(lfptr, temp);
		return;
	}
	if(strncmp(finput,"string",3)==0)
	{
		fscanf(lfptr,"%s\n",&temp);
		for(count = 0; count < MAX_VARS; count++)
		{
			if(strcmp(d_matrix[count].v_name,temp) == 0)
			{
				appastring(lfptr, count);
				return;
			}
		}
		getastring(lfptr, temp);
		return;
	}
	if(strncmp(finput,"constant",3)==0)
	{
		fscanf(lfptr,"%s\n",&temp);
		fscanf(lfptr,"%f\n",&con_value);
		for(count = 0; count < MAX_CONS; count ++)
		{
			if(strlen(constants[count].c_name) < 1 || strcmp(constants[count].c_name,temp) == 0)
			{
				strcpy(constants[count].c_name,temp);
				constants[count].c_value = con_value;
				count = MAX_CONS + 1;
			}
		}
		if(count < MAX_CONS + 1) alert(error(52), alert_time);
		return;
	}
}

void dload (char cstring[MAX_INPUT])
{
	FILE *lfptr;
	int count;
	char filename[NAME_LENGTH];
	char scratch[MAX_INPUT];
	saystatus("Loading Data       ");
	if(oktoquit == FALSE)
	{
		alert(error(11),alert_time);
		return;
	}
	dropall();
	flushempty();
	strcpy(filename,puller(cstring));
	if(strlen(filename) < 1)
	{
		alert(error(48), alert_time);
		return;
	}
	if(!strchr(filename,'.')) strcat(filename,".lds");
	lfptr = fopen(filename,"r");
	if(!lfptr)
	{
		strcpy(scratch,"/usr/lib/ldlas/tut/");
		strcat(scratch,filename);
		lfptr = fopen(scratch,"r");
		if(!lfptr)
		{
			alert(error(12),alert_time);
			return;
		}
	}
	fscanf(lfptr,"%s",&d_matrix_desc);
	fscanf(lfptr,"%i",&missing_ob);
	while(!feof(lfptr))
	{
		getavar(lfptr);
	}
	fclose(lfptr);
	strcpy(d_matrix_desc,add_blanks(d_matrix_desc));
	for(count = 0; count <= MAX_VARS; count++)
	{
		strcpy(d_matrix[count].v_desc,add_blanks(d_matrix[count].v_desc));
	}
	if(strlen(puller(coinput)) > 0) small_error(3);
}

void finput (char cstring[MAX_INPUT])
{
	FILE *lfptr;
	char sinput[MAX_INPUT];
	char **waisted;
	int count, nloads, countt;
	int vartype[MAX_VARS];
	int vnum[MAX_VARS];
	int f_numb[MAX_VARS];
	int i_numb[MAX_VARS];
	int s_numb[MAX_VARS];
	char filename[NAME_LENGTH];
	char tokens[NAME_LENGTH];
	char tokenst[NAME_LENGTH];
	nloads=0;
	saystatus("Inputing from File ");
	strcpy(filename,puller(cstring));
	lfptr = fopen(filename,"r");
	if(!lfptr)
	{
		alert(error(12),alert_time);
		return;
	}
	while(strlen(cstring) > 0)
	{
		strcpy(tokens,puller(cstring));
		if(tokens[0] < 65 || tokens[0] > 122)
		{
			alert(error(53), alert_time);
			return;
		}
		if(tokens[0] > 90 && tokens[0] < 97)
		{
			alert(error(53), alert_time);
			return;
		}
		if(strncmp(tokens,"float",5) == 0)
		{
			nloads++;
			strcpy(tokenst,puller(cstring));
			if(tokenst[0] == '_')
			{
				alert(error(53), alert_time);
				return;
			}
			if(checkname(tokenst)<0)
			{
				alert(error(14),alert_time);
				return;
			}
			vartype[nloads]=1;
			vnum[nloads]=findempty();
			if(vnum[nloads] < 0)
			{
				alert(error(4),alert_time);
				return;
			}
			f_numb[nloads] = typeaug(1,vnum[nloads]);
			fvec[f_numb[nloads]] = (double *) malloc (MAX_OBS * sizeof(double));
			if(!fvec[f_numb[nloads]])
			{
				alert(error(4),alert_time);
				return;
			}
			strcpy(d_matrix[vnum[nloads]].v_name,tokenst);
			d_matrix[vnum[nloads]].fnumb = f_numb[nloads];
			d_matrix[vnum[nloads]].fmem_start=fvec[f_numb[nloads]];
		}
		if(strncmp(tokens,"integer",7) == 0)
		{
			nloads++;
			strcpy(tokenst,puller(cstring));
			if(tokenst[0] == '_')
			{
				alert(error(53), alert_time);
				return;
			}
			if(checkname(tokenst)<0)
			{
				alert(error(14),alert_time);
				return;
			}
			vartype[nloads]=2;
			vnum[nloads]=findempty();
			if(vnum[nloads] < 0)
			{
				alert(error(4),alert_time);
				return;
			}
			i_numb[nloads] = typeaug(2,vnum[nloads]);
			ivec[i_numb[nloads]] = (int *) malloc (MAX_OBS * sizeof(int));
			if(!ivec[i_numb[nloads]])
			{
				alert(error(4),alert_time);
				return;
			}
			strcpy(d_matrix[vnum[nloads]].v_name,tokenst);
			d_matrix[vnum[nloads]].inumb = i_numb[nloads];
			d_matrix[vnum[nloads]].imem_start=ivec[i_numb[nloads]];
		}
		if(strncmp(tokens,"string",5) == 0)
		{
			nloads++;
			strcpy(tokenst,puller(cstring));
			if(tokenst[0] == '_')
			{
				alert(error(53), alert_time);
				return;
			}
			if(checkname(tokens)<0)
			{
				alert(error(14),alert_time);
				return;
			}
			vartype[nloads]=3;
			vnum[nloads]=findempty();
			if(vnum[nloads] < 0)
			{
				alert(error(4),alert_time);
				return;
			}
			s_numb[nloads] = typeaug(3,vnum[nloads]);
			if(s_numb[nloads] + 1 > MAX_SVARS)
			{
				alert(error(5),alert_time);
				return;
			}
			strcpy(d_matrix[vnum[nloads]].v_name,tokenst);
			d_matrix[vnum[nloads]].snumb = s_numb[nloads];
		}
		if(strncmp(tokens,"string",3) != 0 && strncmp(tokens,"integer",3) != 0 && strncmp(tokens,"float",3) != 0)
		{
			nloads++;
			if(checkname(tokens)<0)
			{
				alert(error(14),alert_time);
				return;
			}
			vartype[nloads]=1;
			vnum[nloads]=findempty();
			if(vnum[nloads] < 0)
			{
				alert(error(4),alert_time);
				return;
			}
			f_numb[nloads] = typeaug(1,vnum[nloads]);
			fvec[f_numb[nloads]] = (double *) malloc (MAX_OBS * sizeof(double));
			if(!fvec[f_numb[nloads]])
			{
				alert(error(4),alert_time);
				return;
			}
			strcpy(d_matrix[vnum[nloads]].v_name,tokens);
			d_matrix[vnum[nloads]].fnumb = f_numb[nloads];
			d_matrix[vnum[nloads]].fmem_start=fvec[f_numb[nloads]];
		}
	}
	if(nloads < 1)
	{
		alert(error(44), alert_time);
		return;
	}
	lfptr = fopen(filename,"r");
	if(!lfptr)
	{
		alert(error(12),alert_time);
		return;
	}
	countt=0;
	while(!feof(lfptr))
	{
		for(count = 1; count <= nloads; count++)
		{
			if(vartype[count] == 1)
			{
				fscanf(lfptr,"%lf",&*fvec[f_numb[count]]);
				fvec[f_numb[count]]++;
			}			
			if(vartype[count] == 2)
			{
				fscanf(lfptr,"%i",&*ivec[i_numb[count]]);
				ivec[i_numb[count]]++;
			}			
			if(vartype[count] == 3)
			{
				fscanf(lfptr,"%s",&sinput);
				strcpy(smatrix[s_numb[count]][countt],add_blanks(sinput));
			}
			fscanf(lfptr,"\n");
		}
	countt++; 
	}
	fclose(lfptr);
	for(count = 1;count <= nloads; count++)
	{
		d_matrix[vnum[count]].nobs = countt;
	}
	flushempty();
}

void dlog(char cstring[MAX_INPUT])
{
	int count,countt;
	char filename[NAME_LENGTH];
	char token[MAX_INPUT];
	strcpy(token,puller(cstring));
	if(strncmp(token,"on",2)==0)
	{
		if(existslog == TRUE)
		{
			alert(error(21),alert_time);
			return;
		}
		strcpy(filename,puller(cstring));
		if(strlen(filename) < 1)
		{
			alert(error(48), alert_time);
			return;
		}
		if(!strchr(filename,'.')) strcat(filename,".log");
		log_name = fopen(filename,"a");
		if(!log_name)
		{
			alert(error(8),alert_time);
			return;
		}
		existslog = TRUE;
		fprintf(log_name,"%% \n");
		fprintf(log_name,"%% This file was created/appended by ldlas\n");
		fprintf(log_name,"%% \n");
		if(strlen(puller(coinput)) > 0) small_error(3);
		return;
	}
	if(strncmp(token,"off",3)==0)
	{
		if(existslog == TRUE)
		{
			fclose(log_name);
			existslog = FALSE;
			if(strlen(puller(coinput)) > 0) small_error(3);
			return;
		}
		alert(error(22),alert_time);
		return;
	}
	alert(error(23), alert_time);
}

void dtlog(char cstring[MAX_INPUT])
{
	int count,countt;
	char filename[NAME_LENGTH];
	char token[MAX_INPUT];
	strcpy(token,puller(cstring));
	if(strncmp(token,"on",2)==0)
	{
		if(existstex == TRUE)
		{
			alert(error(24),alert_time);
			return;
		}
		strcpy(filename,puller(cstring));
		if(strlen(filename) < 1)
		{
			alert(error(48), alert_time);
			return;
		}
		if(!strchr(filename,'.')) strcat(filename,".tex");
		tex_name = fopen(filename,"a");
		if(!tex_name)
		{
			alert(error(8),alert_time);
			return;
		}
		fprintf(tex_name,"%% \n");
		fprintf(tex_name,"%% This file was created/appended by ldlas\n");
		fprintf(tex_name,"%% \n");
		existstex = TRUE;
		if(strlen(puller(coinput)) > 0) small_error(3);
		return;
	}
	if(strncmp(token,"off",3)==0)
	{
		if(existstex == TRUE)
		{
			fclose(tex_name);
			existstex = FALSE;
			if(strlen(puller(coinput)) > 0) small_error(3);
			return;
		}
		alert(error(25),alert_time);
		return;
	}
	alert(error(26), alert_time);
}

void fappend(char coinput[MAX_INPUT])
{
	FILE *lfptr;
	char throwout[MAX_INPUT];
	int count;
	char filename[NAME_LENGTH];
	saystatus("Appending Data     ");
	strcpy(filename,puller(coinput));
	if(strlen(filename) < 1)
	{
		alert(error(48), alert_time);
		return;
	}
	if(!strchr(filename,'.')) strcat(filename,".lds");
	lfptr = fopen(filename,"r");
	if(!lfptr)
	{
		alert(error(12),alert_time);
		return;
	}
	fscanf(lfptr,"%s",&throwout);
	fscanf(lfptr,"%i",&count);
	while(!feof(lfptr))
	{
	appavar(lfptr);
	}
	fclose(lfptr);
	for(count = 0; count <= MAX_VARS; count++)
	{
		strcpy(d_matrix[count].v_desc,add_blanks(d_matrix[count].v_desc));
	}
	if(strlen(puller(coinput)) > 0) small_error(3);
}

void letlist(void)
{
	int count;
	int more;
	more = 0;
	if(existslog == TRUE) fprintf(log_name,"\n\n");
	if(existstex == TRUE) 
	{
		fprintf(tex_name,"\n%%\n%%  The following table was created by ldlas\n%%\n");
		fprintf(tex_name,"\n\n\\midinsert\n");
		fprintf(tex_name,"\\bigskip\n");
		fprintf(tex_name,"$$\\hbox{Table of Constants}$$\n");
		fprintf(tex_name,"$$\\vbox{\\offinterlineskip\n");
		fprintf(tex_name,"\\hrule\n");
		fprintf(tex_name,"\\halign{&\\vrule\\hfil#\\hfil&\\strut\\quad\\hfil#\\hfil\\quad\\cr\n");
		fprintf(tex_name,"& Name \t&& Value&\\cr\n");
		fprintf(tex_name,"\\noalign{\\hrule}\n");
	}
	for(count = 0; count < MAX_CONS; count++)
	{
		if(strlen(constants[count].c_name) > 0)
		{
			wprintw(ioarea,"\n %s = %f",constants[count].c_name,constants[count].c_value);
			wrefresh(ioarea);
			if(existslog == TRUE) fprintf(log_name," %s \t %f \n",constants[count].c_name,constants[count].c_value);
			if(existstex == TRUE) fprintf(tex_name,"& %s \t&& %f &\\cr\n",pullone(constants[count].c_name),constants[count].c_value);
			more = check_more(more);
			if(more < 0) count = MAX_CONS;
		}
	}
	if(existstex == TRUE)
	{
		fprintf(tex_name,"}\\hrule}$$\n");
		fprintf(tex_name,"\\bigskip\n");
		fprintf(tex_name,"\\endinsert\n\n");
	}
}

void ldlet(char coinput[MAX_INPUT])
{
	char token[MAX_INPUT];
	char tokent[MAX_INPUT];
	int count;
	strcpy (token,puller(coinput));
	if(strlen(token) < 1)
	{
		letlist();
		return;
	}
	if(token[0] != '_')
	{
		alert(error(50), alert_time);
		return;
	}
	strcpy (tokent,puller(coinput));
	if(strcmp(tokent,"=") == 0)
	{
		strcpy(tokent,puller(coinput));
	}
	if(strlen(tokent) < 1)
	{
		for(count = 0; count < MAX_CONS; count++)
		{
			if(strcmp(token,constants[count].c_name) == 0)
			{
				strcpy(constants[count].c_name,"\0");
				count = MAX_CONS + 1;
			}
		}
		if(count == MAX_CONS)
		{
			alert(error(49), alert_time);
		}
		return;
	}
	for(count = 0; count < MAX_CONS; count++)
	{
		if(strcmp(token,constants[count].c_name) == 0)
		{
			alert(error(51), alert_time);
			return;
		}
	}
	for(count = 0; count < MAX_CONS; count++)
	{
		if(strlen(constants[count].c_name) < 1)
		{
			strcpy(constants[count].c_name,token);
			constants[count].c_value = atof(tokent);
			oktoquit = FALSE;
			if(strlen(puller(coinput)) > 0) small_error(3);
			return;
		}
	}
	alert(error(52), alert_time);
}

void floatappend(int vnumber)
{
	int count,escapeinput,check, f_numb;
	char ininput[MAX_INPUT];
	count = 0;
	escapeinput=0;
	f_numb = d_matrix[vnumber].fnumb;
	fvec[f_numb] = d_matrix[vnumber].fmem_start;
	for(count = 0; count < d_matrix[vnumber].nobs; count++)
	{
		fvec[f_numb]++;
	}
	wprintw(ioarea,"\nEnter floating point values for %s\n",d_matrix[vnumber].v_name);
	do
	{
		wprintw(ioarea,"\n%5i>",(count+1));
		wrefresh(ioarea);
		get_input(ininput);
		if (strcmp(ininput,"\0") == 0) escapeinput = 1;
		else
		{
			*fvec[f_numb] = atof(ininput);
			fvec[f_numb]++;
			count++;
		}
	}
	while(escapeinput == 0);
	d_matrix[vnumber].nobs = count;
	wprintw(ioarea,"\n");
	wrefresh(ioarea);
}

void integerappend(int vnumber)
{
	int count,escapeinput,check, i_numb;
	char ininput[MAX_INPUT];
	count = 0;
	escapeinput=0;
	i_numb = d_matrix[vnumber].inumb;
	ivec[i_numb] = d_matrix[vnumber].imem_start;
	for(count = 0; count < d_matrix[vnumber].nobs; count++)
	{
		ivec[i_numb]++;
	}
	wprintw(ioarea,"\nEnter integer values for %s\n",d_matrix[vnumber].v_name);
	do
	{
		wprintw(ioarea,"\n%5i>",(count+1));
		wrefresh(ioarea);
		get_input(ininput);
		if (strcmp(ininput,"\0") == 0) escapeinput = 1;
		else
		{
			*ivec[i_numb] = atoi(ininput);
			ivec[i_numb]++;
			count++;
		}
	}
	while(escapeinput == 0);
	d_matrix[vnumber].nobs = count;
	wprintw(ioarea,"\n");
	wrefresh(ioarea);
}

void stringappend(int vnumber)
{
	int s_numb,count,escapeinput;
	char ininput[MAX_INPUT];
	count = 0;
	escapeinput=0;
	s_numb = d_matrix[vnumber].snumb;
	count = d_matrix[vnumber].nobs;
	wprintw(ioarea,"\nEnter character strings for %s\n",d_matrix[vnumber].v_name);
	do
	{
		wprintw(ioarea,"\n%5i>",(count+1));
		wrefresh(ioarea);
		get_input(ininput);
		if (strcmp(ininput,"\0") == 0) escapeinput = 1;
		else
		{
			strcpy(smatrix[s_numb][count],ininput);
			count++;
		}
	}
	while(escapeinput == 0);
	d_matrix[vnumber].nobs = count;
	wprintw(ioarea,"\n");
	wrefresh(ioarea);
}

void ldappend(char coinput[MAX_INPUT])
{
	int count, vnumber;
	char token[MAX_INPUT];
	strcpy(token, puller(coinput));
	for(count = 0; count < MAX_VARS; count ++)
	{
		if(strcmp(d_matrix[count].v_name,token) == 0)
		{
			vnumber=count;
			count = MAX_VARS + 1;
		}
	}
	if(count < MAX_VARS + 1)
	{
		alert(error(61), alert_time);
		if(strlen(puller(coinput)) > 0) small_error(3);
		return;
	}
	if(d_matrix[vnumber].fnumb != -1)
	{
		floatappend(vnumber);
		if(strlen(puller(coinput)) > 0) small_error(3);
		return;
	}
	if(d_matrix[vnumber].inumb != -1)
	{
		integerappend(vnumber);
		if(strlen(puller(coinput)) > 0) small_error(3);
		return;
	}
	if(d_matrix[vnumber].snumb != -1)
	{
		stringappend(vnumber);
		if(strlen(puller(coinput)) > 0) small_error(3);
		return;
	}
	alert(error(42), alert_time);
}

